@if "%DEBUG%" == "" @echo off
@rem ##########################################################################
@rem
@rem  syncany startup script for Windows
@rem
@rem ##########################################################################

@rem Set local scope for the variables with windows NT shell
if "%OS%"=="Windows_NT" setlocal

@rem Add default JVM options here. You can also use JAVA_OPTS and SYNCANY_OPTS to pass JVM options to this script.
set DEFAULT_JVM_OPTS="-Dfile.encoding=utf-8"

@rem Read max memory from userconfig.xml #SYNCANY_INCL_1#
set APP_USERCONFIG_FILE=%AppData%\\Syncany\\userconfig.xml

if exist "%APP_USERCONFIG_FILE%" (
  if "%OS%"=="Windows_NT" setlocal ENABLEDELAYEDEXPANSION
  set QT=^"
  for /F "tokens=2,3,5 delims=<=>!QT!" %%a in (%APP_USERCONFIG_FILE%) do ( 
    if "%%a" equ "maxMemory" set APP_MAX_MEMORY=%%b
  )
)

if defined APP_MAX_MEMORY (
  set DEFAULT_JVM_OPTS=%DEFAULT_JVM_OPTS% "-Xmx%APP_MAX_MEMORY%"
) else (
  set DEFAULT_JVM_OPTS=%DEFAULT_JVM_OPTS% "-Xmx2048M"
)

set DIRNAME=%~dp0
if "%DIRNAME%" == "" set DIRNAME=.
set APP_BASE_NAME=%~n0
set APP_HOME=%DIRNAME%..

@rem Find java.exe
if defined JAVA_HOME goto findJavaFromJavaHome

set JAVA_EXE=java.exe
%JAVA_EXE% -version >NUL 2>&1
if "%ERRORLEVEL%" == "0" goto init

echo.
echo ERROR: JAVA_HOME is not set and no 'java' command could be found in your PATH.
echo.
echo Please set the JAVA_HOME variable in your environment to match the
echo location of your Java installation.

goto fail

:findJavaFromJavaHome
set JAVA_HOME=%JAVA_HOME:"=%
set JAVA_EXE=%JAVA_HOME%/bin/java.exe

if exist "%JAVA_EXE%" goto init

echo.
echo ERROR: JAVA_HOME is set to an invalid directory: %JAVA_HOME%
echo.
echo Please set the JAVA_HOME variable in your environment to match the
echo location of your Java installation.

goto fail

:init
@rem Get command-line arguments, handling Windowz variants

if not "%OS%" == "Windows_NT" goto win9xME_args
if "%@eval[2+2]" == "4" goto 4NT_args

:win9xME_args
@rem Slurp the command line arguments.
set CMD_LINE_ARGS=
set _SKIP=2

:win9xME_args_slurp
if "x%~1" == "x" goto execute

set CMD_LINE_ARGS=%*
goto execute

:4NT_args
@rem Get arguments from the 4NT Shell from JP Software
set CMD_LINE_ARGS=%$

:execute
@rem Setup the command line

set CLASSPATH=%APP_HOME%\lib\*;%AppData%\Syncany\plugins\lib\*

@rem Execute syncany
if "%1" == "daemon" goto daemon
"%JAVA_EXE%" %DEFAULT_JVM_OPTS% %JAVA_OPTS% %SYNCANY_OPTS%  -classpath "%CLASSPATH%" org.syncany.Syncany %CMD_LINE_ARGS%
goto afterdaemon

:daemon
rem Embedded Daemon script begins ################################SYNCANY_INCL_3#

set APP_NAME=syncanyd
set APP_USER_DIR=%AppData%\Syncany
set APP_DAEMON_CONTROL=%APP_USER_DIR%\daemon.ctrl
set APP_DAEMON_PIDFILE=%APP_USER_DIR%\daemon.pid
set APP_DAEMON_PIDFILE_TMP=%APP_USER_DIR%\daemon.pid.tmp

if not exist "%APP_USER_DIR%" mkdir "%APP_USER_DIR%" 

rem  Checks if the process is already running using a PID file
rem
rem     Please note that even though this code is incredibly ugly, it took me nearly
rem     five hours to get it right. Unless you are a batch file professional, 
rem
rem                   DO NOT TOUCH THIS CODE!
rem
rem  Steps
rem    1. Read PID file to APID (if file exists)
rem    2. If PID was read successfully (not -1), check if PID is running
rem       by writing error output of wmic command to temp error output file, standard output to NUL
rem    3. Determine size of temp error output file
rem    4. If size of temp error output file is zero (no error), the process is running

set RUNNING=0
set APID=-1

rem 1. Read PID file to APID (if file exists)
if exist %APP_DAEMON_PIDFILE% (
  set /P APID=<%APP_DAEMON_PIDFILE%
) 

rem 2. If PID was read successfully (not -1), check if PID is running
if not "%APID%" == "-1" (  
  wmic process where "ProcessID = %APID%" get processid 2> %APP_DAEMON_PIDFILE_TMP% > NUL
  set /P STDERR=<%APP_DAEMON_PIDFILE_TMP%
  
  rem 3. Determine size of temp error output file
  for /f %%i in ("%APP_DAEMON_PIDFILE_TMP%") do set size=%%~zi
  if not defined size set size=0
  del %APP_DAEMON_PIDFILE_TMP%
)

rem 4. If size of temp error output file is zero (no error), the process is running
if not "%APID%" == "-1" (  
  if "%size%" == "0" set RUNNING=1
)

rem END OF PID FILE STUFF

@if "%2" == "start" goto start
@if "%2" == "stop" goto stop
@if "%2" == "reload" goto reload
@if "%2" == "status" goto status

"%JAVA_EXE%" %DEFAULT_JVM_OPTS% %JAVA_OPTS% %SYNCANY_OPTS%  -classpath "%CLASSPATH%" org.syncany.Syncany %CMD_LINE_ARGS%
goto mainEnd

:stop
if %RUNNING% == 1 (
  echo shutdown >> %APP_DAEMON_CONTROL%
  echo | set /p=Stopping daemon: 
  
  for /l %%i in (1, 1, 10) do (
    if exist %APP_DAEMON_PIDFILE% (
	  echo | set /p=.
	  timeout /t 1 /nobreak > NUL
	) 
  )
  
  if exist %APP_DAEMON_PIDFILE% (
    echo  Failed. 
	exit /b 1
  ) else (
    echo  %APP_NAME%.
  )
) else (
  echo Stopping daemon: %APP_NAME% not running
)
goto mainEnd

:reload
if %RUNNING% == 1 (
  echo reload >> %APP_DAEMON_CONTROL%
  echo | set /p=Reloading daemon: %APP_NAME%.
) else (
  echo Reloading daemon: %APP_NAME% not running
)
goto mainEnd

:status
if %RUNNING% == 1 (
  echo Checking daemon: %APP_NAME% running with pid %APID%
) else (
  echo Checking daemon: %APP_NAME% not running
)
goto mainEnd

:start
@rem Find java.exe
if defined JAVA_HOME goto findJavaFromJavaHome

set JAVA_EXE=javaw.exe
%JAVA_EXE% -version >NUL 2>&1
if "%ERRORLEVEL%" == "0" goto init

echo.
echo ERROR: JAVA_HOME is not set and no 'java' command could be found in your PATH.
echo.
echo Please set the JAVA_HOME variable in your environment to match the
echo location of your Java installation.

exit /b 1

:findJavaFromJavaHome
set JAVA_HOME=%JAVA_HOME:"=%
set JAVA_EXE=%JAVA_HOME%/bin/javaw.exe

if exist "%JAVA_EXE%" goto init

echo.
echo ERROR: JAVA_HOME is set to an invalid directory: %JAVA_HOME%
echo.
echo Please set the JAVA_HOME variable in your environment to match the
echo location of your Java installation.

exit /b 1

:init
if %RUNNING% == 1 (
  echo Starting daemon: %APP_NAME% already running with pid %APID%
  goto mainEnd
) 

set CLASSPATH=%APP_HOME%\lib\*;%AppData%\Syncany\plugins\lib\*

echo | set /p=Starting daemon: .
start "" /b "%JAVA_EXE%" %DEFAULT_JVM_OPTS% %JAVA_OPTS% -classpath "%CLASSPATH%" org.syncany.Syncany daemon run

for /l %%i in (1, 1, 10) do (
  set "pid_and_ctrl_exists="
  if exist %APP_DAEMON_PIDFILE% if exist %APP_DAEMON_CONTROL% set pid_and_ctrl_exists=1
  
  if not defined pid_and_ctrl_exists (
    echo | set /p=.
    timeout /t 1 /nobreak > NUL
  ) 
)

set "pid_and_ctrl_exists="
if exist %APP_DAEMON_PIDFILE% if exist %APP_DAEMON_CONTROL% set pid_and_ctrl_exists=1
  
if not defined pid_and_ctrl_exists (
  echo  Failed. 
  exit /b 1
) else (
  echo  %APP_NAME%.
  goto mainEnd
)

rem Embedded Daemon script ends #########################################################


:afterdaemon

@rem Delete plugin JARs #SYNCANY_INCL_2#
SET PURGEFILE=%AppData%\Syncany\purgefile
if exist %PURGEFILE% (
  @for /f %%b in (%PURGEFILE%) do del /q "%%b" 2>NUL
  del /q %PURGEFILE% 2>NUL
)

@rem Reinstall plugins after update removal #SYNCANY_INCL_4#
SET UPDATEFILE=%AppData%\Syncany\updatefile
if exist %UPDATEFILE% (
  @for /f %%b in (%UPDATEFILE%) do (
    <nul set /p =Updating %%b... 
    "%JAVA_EXE%" %DEFAULT_JVM_OPTS% %JAVA_OPTS% %SYNCANY_OPTS%  -classpath "%CLASSPATH%" org.syncany.Syncany plugin install %%b -m 2>NUL)
  del /q %UPDATEFILE% 2>NUL
)

:end
@rem End local scope for the variables with windows NT shell
if "%ERRORLEVEL%"=="0" goto mainEnd

:fail
rem Set variable SYNCANY_EXIT_CONSOLE if you need the _script_ return code instead of
rem the _cmd.exe /c_ return code!
if  not "" == "%SYNCANY_EXIT_CONSOLE%" exit 1
exit /b 1

:mainEnd
if "%OS%"=="Windows_NT" endlocal

:omega
